﻿using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows.Forms;

namespace WinFormsApp1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        public static SynchronizationContext s_ctxThread;

        private void Form1_Load(object sender, EventArgs e)
        {
            s_ctxThread = WindowsFormsSynchronizationContext.Current;
            LoadSettings();

            _lbl12CycleTime.Text = $"하루 주기: {(12 * 60 * 2.916666666666667) / 60 * 2} 분";

            _timer.Start();
        }

        public void ChangeWindowStatus(bool tray)
        {
            if (tray == true)
            {
                this.WindowState = FormWindowState.Minimized;
                this.ShowInTaskbar = false;
            }
            else
            {
                this.Show();
                this.ShowInTaskbar = true;
                this.WindowState = FormWindowState.Normal;
                SafeNative.BringWindowToTop(this.Handle);
                SafeNative.SetForegroundWindow(this.Handle);
            }
        }

        bool _lastCondition = false;
        DateTime _elapsedOld = DateTime.MinValue;

        private void _timer_Tick(object sender, EventArgs e)
        {
            double now = TimeUtil.CurrentEorzeaTime;

            int hour = (int)((now % (3600 * 24)) / 3600.0);
            int minute = (int)((now % 3600) / 60.0);

            if (UserSettings.Default.Alarm == false)
            {
                return;
            }

            int alarmHour = UserSettings.Default.AlarmTime1.Hour;
            
            bool conditionMet = (hour == alarmHour || (_chk12.Checked == true && (hour % 12) == alarmHour))
                            && minute == UserSettings.Default.AlarmTime1.Minute;

            if (conditionMet != _lastCondition)
            {
                if (conditionMet == true)
                {
                    ChangeWindowStatus(false);
                    ThreadPool.QueueUserWorkItem(new WaitCallback((object target) =>
                    {
                        ShakeMe();
                    }));
                }
            }

            _lastCondition = conditionMet;
            this.Text = $"{hour:00}:{minute:00}";
        }

        private void SaveSettings()
        {
            UserSettings.Default.Alarm = _chkAlarm.Checked;
            UserSettings.Default.After12 = _chk12.Checked;

            int hour = int.Parse(_txtAlarm1Hour.Text);
            int minute = int.Parse(_txtAlarm1Minute.Text);

            UserSettings.Default.AlarmTime1 = new DateTime(1, 1, 1, hour, minute, 0);

            UserSettings.Default.Save();
            _btnSave.Enabled = false;
        }

        private void LoadSettings()
        {
            _chkAlarm.Checked = UserSettings.Default.Alarm;
            _chk12.Checked = UserSettings.Default.After12;
            _txtAlarm1Hour.Text = UserSettings.Default.AlarmTime1.Hour.ToString("00");
            _txtAlarm1Minute.Text = UserSettings.Default.AlarmTime1.Minute.ToString("00");
            _btnSave.Enabled = false;
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            SaveSettings();
        }

        private void _chkAlarm_CheckedChanged(object sender, EventArgs e)
        {
            _btnSave.Enabled = true;
        }

        private void _txt_TextChanged(object sender, EventArgs e)
        {
            TextBox target = sender as TextBox;

            if (int.TryParse(target.Text, out int _) == false)
            {
                return;
            }

            _btnSave.Enabled = true;
        }

        // https://stackoverflow.com/questions/463299/how-do-i-make-a-textbox-that-only-accepts-numbers
        private void _txt_NumberOnly_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData != Keys.Back)
            {
                e.SuppressKeyPress = !int.TryParse(Convert.ToString((char)e.KeyData), out int _);
            }
        }

        // https://www.sysnet.pe.kr/2/0/966
        void ShakeMe()
        {
            Point myLoc = this.Location;
            Point myDefaultLoc = this.Location;

            s_ctxThread.Post(new SendOrPostCallback((object target) =>
            {
                this.TopMost = true;
                this.TopMost = false;
            }), null);

            for (int i = 0; i < 20; i++)
            {
                for (int x = 0; x <= 4; x++)
                {
                    switch (x)
                    {
                        case 0:
                            myLoc.X = myDefaultLoc.X + 10;
                            break;
                        case 1:
                            myLoc.X = myDefaultLoc.X - 10;
                            break;
                        case 2:
                            myLoc.Y = myDefaultLoc.Y + 10;
                            break;
                        case 3:
                            myLoc.Y = myDefaultLoc.Y - 10;
                            break;
                        case 4:
                            myLoc = myDefaultLoc;
                            break;
                    }

                    Thread.Sleep(10);

                    s_ctxThread.Send(new SendOrPostCallback((object target) =>
                    {
                        this.Location = myLoc;
                    }), null);

                }
            }

            s_ctxThread.Send(new SendOrPostCallback((object target) =>
            {
                this.Location = myDefaultLoc;
            }), null);
        }

    }
}
